@@ -24,13 +24,12 @@ from threading import local
 from zope.security.interfaces import NoInteraction
 
 # import local interfaces
-from ztfy.alchemy.interfaces import IAlchemyEngineUtility
+from ztfy.alchemy.interfaces import IAlchemyEngineUtility, REQUEST_SESSION_KEY
 
 # import Zope3 packages
 from zope.container.contained import Contained
 from zope.interface import implements
-from zope.component import adapter, getUtility
-from zope.publisher.interfaces import IEndRequestEvent
+from zope.component import getUtility
 from zope.schema.fieldproperty import FieldProperty
 
 # import local packages
@@ -122,18 +121,7 @@ def getEngine(engine):
     return engine
 
 
-class SessionsStorage(local):
-    """Thread local sessions storage"""
-
-    engines = {}
-
-    def clear(self):
-        self.engines.clear()
-
-sessions = SessionsStorage()
-
-
-def getSession(engine, join=True, status=STATUS_ACTIVE, request=None, alias=None):
+def getSession(engine, join=True, status=STATUS_ACTIVE, request=None, alias=None, twophase=True):
     """Get a new SQLAlchemy session
     
     Session is stored in request and in a sessions storage"""
@@ -143,44 +131,34 @@ def getSession(engine, join=True, status=STATUS_ACTIVE, request=None, alias=None
             request = getRequest()
         except NoInteraction:
             pass
-    if alias is None:
+    if not alias:
         alias = engine
     if request is not None:
-        session_data = getRequestData('ztfy.alchemy.session', request, {})
+        session_data = getRequestData(REQUEST_SESSION_KEY, request, {})
         session = session_data.get(alias)
     if session is None:
-        if alias in sessions.engines:
-            session = sessions.engines[alias]
-            if status != STATUS_READONLY:
-                _SESSION_STATE[id(session)] = status
-        if session is None:
-            _engine = getEngine(engine)
-            factory = scoped_session(sessionmaker(bind=_engine, twophase=True, extension=ZopeTransactionExtension()))
-            session = factory()
-            if join:
-                join_transaction(session, initial_state=status)
-            sessions.engines[alias] = session
+        _engine = getEngine(engine)
+        factory = scoped_session(sessionmaker(bind=_engine, twophase=twophase, extension=ZopeTransactionExtension()))
+        session = factory()
+        if join:
+            join_transaction(session, initial_state=status)
+        if status != STATUS_READONLY:
+            _SESSION_STATE[id(session)] = status
         if (request is not None) and (session is not None):
             session_data[alias] = session
-            setRequestData('ztfy.alchemy.session', session_data, request)
+            setRequestData(REQUEST_SESSION_KEY, session_data, request)
     return session
 
 
-def getUserSession(engine, status=STATUS_ACTIVE, request=None):
+def getUserSession(engine, join=True, status=STATUS_ACTIVE, request=None, alias=None, twophase=True):
     """Shortcut function to get user session"""
     if isinstance(engine, basestring):
-        session = getSession(engine, status=status, request=request)
+        session = getSession(engine, join=join, status=status, request=request, alias=alias, twophase=twophase)
     else:
         session = engine
     return session
 
 
-@adapter(IEndRequestEvent)
-def handleEndRequest(event):
-    """Clear thread's sessions on request end"""
-    sessions.clear()
-
-
 class MetadataManager(object):
     """A manager for metadata management, to be able to use the same table name
     in different databases
